PROGRAM ex100_get_dimension
  IMPLICIT NONE
! +++ Sample read program for MOD04_L2 & MYD04_L2 (Level 2; 10 km at nadir)
!
!     2nd release: 2015-10-02 (NIES, Japan)
!     1st release: 2009-11-02 (CEReS, Chiba Univ., Japan)
!       M. Hayasaki
!         http://www.hysk.sakura.ne.jp/
  INTEGER swopen, swinqswath, swattach, swdetach, swclose
  INTEGER sd_id, sw_id, nswath, statn

  CHARACTER(LEN=256)  :: l2_file
  CHARACTER(LEN=256)  :: swathname  ! Ex. "mod04"
  CHARACTER(LEN=64)   :: dim_name   ! Ex. "Cell_Along_Swath", "Cell_Across_Swath", ...
  INTEGER dim_id, dim_size, data_type, num_attr, nchar

! +++ Read input filename (l2_file) from the 1st argument ($1). 
  CALL GETARG(1, l2_file)

!!! Open HDF-EOS Level-2 (swath) file using "swopen()" function
  sd_id = swopen(TRIM(l2_file), 1)

!!! Get number of swath(s): nswath = 1 if you use MOD04_L2 file.
  nswath = swinqswath(TRIM(l2_file), swathname, nchar)
  WRITE(*, '(4x, a, i5, a, i5)') "No. of swaths = ", nswath, "; nchar = ", nchar
  WRITE(*, '(4x, a)') "swath name = "//TRIM(swathname)

  ! Ready to read swath data (Attach "swath ID" using "swattach()" function)
  sw_id = swattach(sd_id, swathname)

  WRITE(*, '(4x,a,2x,  2i10,2x, a)') &
      "File ID, swath ID, filename = ", sd_id, sw_id, TRIM(l2_file)

  !!! Get a size of dimensions (Use invariant DimensionName)
  CALL test031_swdiminfo(sw_id)     ! IN

  !! Detach swath ID, close file.
  statn = swdetach(sw_id)
  statn = swclose(sd_id)

END PROGRAM ex100_get_dimension
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
SUBROUTINE test031_swdiminfo(sw_id)   ! IN
  INTEGER,          INTENT(IN)  :: sw_id    ! swath ID

  CHARACTER(LEN=256) :: dim_name    ! DimensionName
  INTEGER  :: swdiminfo, dim_size

!!! Sample dimension name. See a head of "ncdump -h hoge.hdf"
  dim_name = 'Cell_Across_Swath'
  dim_size = swdiminfo(sw_id, TRIM(dim_name))
  WRITE(*, '(6x,a, i16)') "dimsize of "//TRIM(dim_name)//" = ", dim_size

  dim_name = 'Cell_Along_Swath'
  dim_size = swdiminfo(sw_id, TRIM(dim_name))
  WRITE(*, '(6x,a, i16)') "dimsize of "//TRIM(dim_name)//" = ", dim_size

  dim_name = 'QA_Byte_Land'
  dim_size = swdiminfo(sw_id, TRIM(dim_name))
  WRITE(*, '(6x,a, i16)') "dimsize of "//TRIM(dim_name)//" = ", dim_size

END SUBROUTINE test031_swdiminfo
