PROGRAM ex300_get_AOD
  USE settings_MODIS_sample
  USE mod_open_hdfeos,     ONLY : open_L2_hdf
  USE mod_sample_decode_L2, ONLY : set_dimsize_MOD04_L2,  &
        dump_lonlat, close_sds, deep_blue_extract_sds_MOD04
  IMPLICIT NONE

! +++ Set parameters and define variables (Level 2; 10 km at nadir)

  TYPE(baseparam) :: baseinfo
  TYPE(swath_param) :: swath_info
  INTEGER sd_id, sw_id
  INTEGER iunit, ounit
  INTEGER nx, ny, ix, iy, dx_step, dy_step
  INTEGER dim_size_along_swath, dim_size_across_swath
  CHARACTER(LEN=256) :: l2_file

  REAL, ALLOCATABLE :: L2_xlon(:, :), L2_ylat(:, :)
  REAL, ALLOCATABLE :: L2_AOD(:, :)
  INTEGER, ALLOCATABLE :: L2_AOD_flag(:, :)

! +++ Return the input filename (l2_file) from the 1st argument ($1). 
!   return sd_id (file ID) and swid (swath ID)
  CALL open_L2_hdf(sd_id, sw_id, l2_file)   ! OUT

  WRITE(*, '(4x,a,2x,  2i10,2x, a)') &
      "File ID, swath ID, filename = ", sd_id, sw_id, TRIM(l2_file)
  swath_info%sd_id = sd_id    ! file ID
  swath_info%sw_id = sw_id    ! swath ID
  swath_info%ifile = TRIM(l2_file)

  CALL set_dimsize_MOD04_L2(sw_id,  &   ! IN
        dim_size_along_swath, dim_size_across_swath)    ! OUT (Ex. 203, 135 in MOD04_L2)
  nx = dim_size_across_swath
  ny = dim_size_along_swath  

  ALLOCATE ( L2_xlon(nx, ny), L2_ylat(nx, ny) )
  ALLOCATE ( L2_AOD(nx, ny), L2_AOD_flag(nx, ny) )

  CALL dump_lonlat(sw_id, nx, ny,   &   ! IN
        L2_xlon, L2_ylat)     ! OUT

  CALL deep_blue_extract_sds_MOD04(nx, ny, swath_info,   &   ! IN
        L2_AOD, L2_AOD_flag)     ! OUT

  dy_step = INT(ny / 10)
  dx_step = INT(nx / 10)
  DO iy = 1, ny, dy_step      ! along-swath
    DO ix = 1, nx, dx_step    ! across-swath
      WRITE(*, '(2x, a, 2f12.4, a,1x, f12.4,1x, i8)')  &
          "DEBUG MAIN: lon, lat = ",  & 
          L2_xlon(ix, iy), L2_ylat(ix, iy),   &
          "; AOD, AOD_flag = ",   &
          L2_AOD(ix, iy), L2_AOD_flag(ix, iy)
    END DO  ! ix
  END DO  ! iy

  DEALLOCATE ( L2_xlon, L2_ylat )
  DEALLOCATE ( L2_AOD, L2_AOD_flag )
  CALL close_sds(sw_id, sd_id)

END PROGRAM ex300_get_AOD
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
