MODULE mod_open_hdfeos

  IMPLICIT NONE
  PRIVATE
  INTEGER, PARAMETER :: MAX_FILE_NAME = 256
  INTEGER, PARAMETER :: DFACC_RDONLY = 1

  PUBLIC open_L3_hdf
  PUBLIC open_L2_hdf, open_L2_hdfeos_intent_in
  PUBLIC open_L1B_hdfeos_intent_in

! SUBROUTINE open_L2_hdfeos_intent_in(l2_file,  &   ! IN
!       sd_id, sw_id)   ! OUT

! SUBROUTINE open_L2_hdf(sd_id, sw_id, l2_file)   ! OUT

CONTAINS
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE open_L3_hdf(sd_id)   ! OUT
  INTEGER, INTENT(OUT) :: sd_id
  CHARACTER(LEN=MAX_FILE_NAME) :: l3_file
  INTEGER sfstart

!     Read the input filename.
  CALL GETARG(1, l3_file)
  IF (l3_file  ==  ' ') THEN
    PRINT *, 'Usage: read_qscat3 <Level 3 file>'
    STOP
  END IF
  WRITE(*, *)
  WRITE(*, '(a, a)') "FILENAME = ", TRIM(l3_file)
  WRITE(*, '(2x,a, i8)') "Length of filename = ", LEN_TRIM(l3_file)

!     Open the HDF input file and initiate the SD interface
  sd_id = sfstart(l3_file, DFACC_RDONLY)

  END SUBROUTINE open_L3_hdf
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE open_L2_hdfeos_intent_in(l2_file,  &   ! IN
        sd_id, sw_id)   ! OUT
  CHARACTER(LEN=*), INTENT(IN) :: l2_file
  INTEGER, INTENT(OUT) :: sd_id, sw_id

  INTEGER swopen, swinqswath, swattach
  INTEGER swclose, swdetach, statn
  CHARACTER(LEN=max_file_name) swathname   ! Name of swath
  INTEGER nchar           ! Number of characters
  INTEGER nswath          ! Number of swaths

!     Read the input filename.
  IF (LEN_TRIM(l2_file)  == 0) THEN
    PRINT *, 'Usage: dump_modis_L2 <MODIS Level 2 file>'
    STOP
  END IF
! WRITE(*, '(a)') "FILENAME = "//TRIM(l2_file)
! WRITE(*, '(2x,a, i8)') "Length of filename = ", LEN_TRIM(l2_file)

!     Open the HDF input file and initiate the SD interface
  sd_id = swopen(TRIM(l2_file), DFACC_RDONLY)
  IF (sd_id == -1) THEN
    print *, "Error ", sd_id, " opening file ", TRIM(l2_file)
    statn = swclose(sd_id)
!   RETURN
!   stop
  END IF

! Get name of swath(s)
  nswath = swinqswath(TRIM(l2_file), swathname, nchar)
  IF (nswath /= 1) THEN
    print *, "swinqswath found ", nswath, & 
        " swaths for file ", TRIM(l2_file), " Need exactly 1"
    stop
  END IF

  ! There's exactly one swath.  Make sure it is the right one.
! IF (swathname /= 'mod04') THEN
!   print *, "Error: bad swath name ", TRIM(swathname),   &
!       " in file ", TRIM(l2_file)
!   print *, "Expected mod04"
!   stop
! END IF
  
  ! Attach to (open) the one swath.
  sw_id = swattach(sd_id, swathname)
  IF (sw_id == -1) THEN
    print *, "Failed to attach to swath ", TRIM(swathname),     &
        " in file ", TRIM(l2_file)
    statn = swdetach(sw_id)
!   RETURN
!   stop
  END IF

  END SUBROUTINE open_L2_hdfeos_intent_in
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE open_L2_hdf(sd_id, sw_id, l2_file)   ! OUT
  INTEGER, INTENT(OUT) :: sd_id   ! HDF-EOS file ID
  INTEGER, INTENT(OUT) :: sw_id  ! HDF-EOS swath ID
  CHARACTER(LEN=MAX_FILE_NAME), INTENT(OUT) :: l2_file
  INTEGER sfstart

  INTEGER nchar           ! Number of characters
  CHARACTER(LEN=max_file_name) swathname   ! Name of swath
  INTEGER nswath          ! Number of swaths

  INTEGER swopen, swinqswath, swattach
  INTEGER swrdfld, swrdattr
  INTEGER swdetach, swclose

!     Read the input filename.
  CALL GETARG(1, l2_file)
  IF (l2_file  ==  ' ') THEN
    PRINT *, 'Usage: dump_modis_L2 <MODIS Level 2 file>'
    STOP
  END IF
  WRITE(*, *)
  WRITE(*, '(a, a)') "FILENAME = ", TRIM(l2_file)
  WRITE(*, '(2x,a, i8)') "Length of filename = ", LEN_TRIM(l2_file)

!     Open the HDF input file and initiate the SD interface
  sd_id = swopen(TRIM(l2_file), 1)
  IF (sd_id == -1) THEN
    print *, "Error ", sd_id, " opening file ", TRIM(l2_file)
    stop
  END IF
  WRITE(*, '(a, i10)') "sd_id = ", sd_id

! Get name of swath(s)
  nswath = swinqswath(TRIM(l2_file), swathname, nchar)
  IF (nswath /= 1) THEN
    print *, "swinqswath found ", nswath, &
        " swaths for file ", TRIM(l2_file), " Need exactly 1"
    stop
  END IF
  WRITE(*, '(4x, a)') "swathname = "//TRIM(swathname)


  ! There's exactly one swath.  Make sure it is the right one.
! IF (swathname /= 'mod04') THEN
!   print *, "Error: bad swath name ", TRIM(swathname),   &
!       " in file ", TRIM(l2_file)
!   print *, "Expected mod04"
!   stop
! END IF

  ! Attach to (open) the one swath.
  sw_id = swattach(sd_id, swathname)
  IF (sw_id == -1) THEN
    print *, "Failed to attach to swath ", TRIM(swathname),     &
        " in file ", TRIM(l2_file)
    stop
  END IF

  END SUBROUTINE open_L2_hdf
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE open_L1B_hdfeos_intent_in(L1B_file,  &   ! IN
        sd_id, sw_id)   ! OUT
  CHARACTER(LEN=*), INTENT(IN) :: L1B_file
  INTEGER, INTENT(OUT) :: sd_id, sw_id

  INTEGER swopen, swinqswath, swattach
  CHARACTER(LEN=max_file_name) swathname   ! Name of swath
  INTEGER nchar           ! Number of characters
  INTEGER nswath          ! Number of swaths

!     Read the input filename.
  IF (LEN_TRIM(L1B_file)  == 0) THEN
    PRINT *, 'Usage: dump_L1B_RGB_Asia <Level 1B calibrated radiance file>'
    STOP
  END IF
! WRITE(*, '(a)') "FILENAME = "//TRIM(L1B_file)
! WRITE(*, '(2x,a, i8)') "Length of filename = ", LEN_TRIM(L1B_file)

!     Open the HDF input file and initiate the SD interface
  sd_id = swopen(TRIM(L1B_file), DFACC_RDONLY)
  IF (sd_id == -1) THEN
    print *, "Error ", sd_id, " opening file ", TRIM(L1B_file)
    stop
  END IF

! Get name of swath(s)
  nswath = swinqswath(TRIM(L1B_file), swathname, nchar)
  IF (nswath /= 1) THEN
    print *, "swinqswath found ", nswath, & 
        " swaths for file ", TRIM(L1B_file), " Need exactly 1"
    stop
  END IF

  ! There's exactly one swath.  Make sure it is the right one.
! IF (swathname /= 'mod04') THEN
!   print *, "Error: bad swath name ", TRIM(swathname),   &
!       " in file ", TRIM(L1B_file)
!   print *, "Expected mod04"
!   stop
! END IF
  
  ! Attach to (open) the one swath.
  sw_id = swattach(sd_id, swathname)
  IF (sw_id == -1) THEN
    print *, "Failed to attach to swath ", TRIM(swathname),     &
        " in file ", TRIM(L1B_file)
    stop
  END IF

  END SUBROUTINE open_L1B_hdfeos_intent_in
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
END MODULE mod_open_hdfeos
