MODULE mod_sample_decode_L2
  USE settings_MODIS_sample
  IMPLICIT NONE
  PRIVATE

  PUBLIC get_scale_offset

  PUBLIC test031_swdiminfo

  PUBLIC set_dimsize_MOD04_L2
! SUBROUTINE set_dimsize_MOD04_L2(sw_id,  &  ! IN
!     dim_size_along_swath, dim_size_across_swath)    ! OUT (Ex. 203, 135 in MOD04_L2)

  PUBLIC deep_blue_extract_sds_MOD04
! SUBROUTINE deep_blue_extract_sds_MOD04(nx, ny, swath_info,  &   ! IN
!     L2_AOD, L2_AOD_flag)   ! OUT

  PUBLIC sfc_albedo_extract_sds_MOD04
! SUBROUTINE sfc_albedo_extract_sds_MOD04(nx, ny, swath_info,  &   ! IN
!     L2_albedo)   ! OUT

  PUBLIC dump_lonlat
! SUBROUTINE dump_lonlat(sw_id, nx, ny,   &   ! IN
!       out_lon, out_lat)   ! OUT

  PUBLIC close_sds
! SUBROUTINE close_sds(sw_id, sd_id)    ! IN


  INTEGER  ix, iy
  INTEGER  jx, jy

  INTEGER statn           ! HDF-EOS status. 0 for success
  INTEGER start(10) /0,0,0,0,0, 0,0,0,0,0/
                            ! start of each dimensions for Swath I/O
                            ! 0 => start with first element
  INTEGER stride(10)/1,1,1,1,1, 1,1,1,1,1/
                            ! stride of each dimensions for Swath I/O
                            ! 1 => use every element
  INTEGER edge(10)        ! size of each dimension for swath I/O
                            ! will be set for each individual read

! REAL            dummy(CELL_ACROSS_SWATH,   CELL_ALONG_SWATH)   ! for "float" variables
! BYTE            i1dummy(CELL_ACROSS_SWATH, CELL_ALONG_SWATH)   ! for "byte" variables

  CHARACTER(LEN=4) :: cHHMM

! +++ To retrieve scale factor and offset
  CHARACTER(LEN=max_len)    ::  input_file
  CHARACTER(LEN=max_len)    ::  target_field_name
  CHARACTER(LEN=max_len)    ::  target_QA_name
  REAL(KIND=8)   dscale_factor, doffset, dmissing

  REAL    scale_factor, offset

CONTAINS
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
SUBROUTINE test031_swdiminfo(sw_id)   ! IN
  INTEGER,          INTENT(IN)  :: sw_id    ! swath ID

  CHARACTER(LEN=256) :: dim_name    ! DimensionName
  INTEGER  :: swdiminfo, dim_size

!!! Sample dimension name. See a head of "ncdump -h hoge.hdf"
  dim_name = 'Cell_Across_Swath'
  dim_size = swdiminfo(sw_id, TRIM(dim_name))
  WRITE(*, '(6x,a, i16)') "dimsize of "//TRIM(dim_name)//" = ", dim_size

  dim_name = 'Cell_Along_Swath'
  dim_size = swdiminfo(sw_id, TRIM(dim_name))
  WRITE(*, '(6x,a, i16)') "dimsize of "//TRIM(dim_name)//" = ", dim_size

  dim_name = 'QA_Byte_Land'
  dim_size = swdiminfo(sw_id, TRIM(dim_name))
  WRITE(*, '(6x,a, i16)') "dimsize of "//TRIM(dim_name)//" = ", dim_size

END SUBROUTINE test031_swdiminfo
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE set_dimsize_MOD04_L2(sw_id,  &  ! IN
      dim_size_along_swath, dim_size_across_swath)    ! OUT (Ex. 203, 135 in MOD04_L2)

  INTEGER,          INTENT(IN)  :: sw_id      ! swath ID
  INTEGER,          INTENT(OUT) :: dim_size_along_swath
  INTEGER,          INTENT(OUT) :: dim_size_across_swath

!!! Ex. MYD04_L2 (Level-2, aerosol product), default
!   Cell_Along_Swath  = 203
!   Cell_Across_Swath = 135
!     -> Some files have different sizes.
  INTEGER  i, dim_size
  INTEGER  swdiminfo

  CHARACTER(LEN=256) :: dim_name

    dim_name = "Cell_Along_Swath"
    dim_size_along_swath = swdiminfo(sw_id, TRIM(dim_name))

    dim_name = "Cell_Across_Swath"
    dim_size_across_swath = swdiminfo(sw_id, TRIM(dim_name))


  WRITE(*, '(2x, a, i8)')  &
      "Cell size: Cell_Along_Swath  = ", dim_size_along_swath
  WRITE(*, '(2x, a, i8)')  &
      "Cell size: Cell_Across_Swath = ", dim_size_across_swath

  END SUBROUTINE set_dimsize_MOD04_L2
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE deep_blue_extract_sds_MOD04(nx, ny, swath_info,  &   ! IN
      L2_AOD, L2_AOD_flag)   ! OUT
! +++ Dump "aerosol product" (MOD04 or MYD04 data)
  INTEGER, INTENT(IN)  ::  nx, ny     ! across_swath, along_swath
  TYPE(swath_param), INTENT(IN) :: swath_info
  REAL(4), INTENT(OUT) ::  L2_AOD(nx, ny)
  INTEGER(4), INTENT(OUT) ::  L2_AOD_flag(nx, ny)

  INTEGER   swrdfld, swdetach, swclose
  INTEGER   stdout_interval
  INTEGER   sw_id, sd_id
  REAL(4)   a1, a2, a3
!!  REAL(4), ALLOCATABLE ::  L2_xlon(:, :), L2_ylat(:, :)
  REAL(4) ::  L2_xlon(nx, ny), L2_ylat(nx, ny)

!!  INTEGER(KIND=2), ALLOCATABLE :: i2dummy(:, :)
  INTEGER(KIND=2) :: i2dummy(nx, ny)

  CHARACTER(LEN=max_len)  ::  var_name
  CHARACTER(LEN=max_len)  ::  QA_var_name

  CHARACTER(LEN=max_len)    ::  input_file
  CHARACTER(LEN=max_len)    ::  sub_name

  sub_name = "SUB deep_blue_extract_sds_MOD04"

  sd_id = swath_info%sd_id
  sw_id = swath_info%sw_id
  cHHMM = swath_info%cHHMM
  input_file = swath_info%ifile

! +++ Set basic parameters for 2-D array: dat(nx, ny)
! nx = swath_info%num_across_swath
! ny = swath_info%num_along_swath

! nx = across_swath (E-W), ny = along_swath (N-S)
  CALL dump_lonlat(sw_id, nx, ny,   &   ! IN
        L2_xlon, L2_ylat)   ! OUT

! +++ Aerosol Optical Depth (AOD; 2-byte integer; scale_factor = 0.001)
  edge(1) = nx
  edge(2) = ny

!!!  target_field_name = "Optical_Depth_Land_And_Ocean"
  target_field_name = "AOD_550_Dark_Target_Deep_Blue_Combined"
  statn = swrdfld(sw_id, TRIM(target_field_name), start, stride, edge, i2dummy)
  IF (statn /= 0) CALL swrdfld_err_msg(statn, TRIM(target_field_name) )

!!! Convert into float data for AOD (normal range = 0 - 3)
  CALL get_scale_offset(TRIM(input_file),  TRIM(target_field_name),    &    ! IN
      dscale_factor, doffset, dmissing)   ! OUT
  CALL convert_short2float(nx, ny, dscale_factor, doffset, i2dummy,    &    ! IN
      L2_AOD)   ! OUT

!!! Quality flag (use flag=2, 3 only)
  target_QA_name = "AOD_550_Dark_Target_Deep_Blue_Combined_QA_Flag"
  statn = swrdfld(sw_id, TRIM(target_QA_name), start, stride, edge, i2dummy)
  IF (statn /= 0) CALL swrdfld_err_msg(statn, TRIM(target_QA_name) )

  L2_AOD_flag(:, :) = i2dummy(:, :)


! +++ TEST output
  stdout_interval = 20
  DO iy = 1, ny, stdout_interval
    ix = 1 + nx / 2    ! nadir + 1 only
    a1 = L2_AOD(ix, iy)

    IF ( a1 > -900 ) THEN
      WRITE(*, '(a, a,2x, 2i4, a, 2f12.3, f12.3)') &
          TRIM(sub_name)//": HHMM, ix, iy = ", cHHMM, ix, iy,    &
          "; lon, lat, AOD = ", L2_xlon(ix, iy), L2_ylat(ix, iy),   &
          L2_AOD(ix, iy)
    END IF

  END DO    ! iy

END SUBROUTINE deep_blue_extract_sds_MOD04
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE sfc_albedo_extract_sds_MOD04(nx, ny, swath_info,  &   ! IN
      L2_albedo)   ! OUT
! +++ Dump "aerosol product" (MOD04 or MYD04 data)
  INTEGER, INTENT(IN)  ::  nx, ny     ! across_swath, along_swath
  TYPE(swath_param), INTENT(IN) :: swath_info
  REAL, INTENT(OUT) :: L2_albedo(nx, ny)

  !! local variables
  INTEGER(KIND=2) :: i2dummy(nx, ny)
  REAL :: L2_xlon(nx, ny), L2_ylat(nx, ny)

  INTEGER   swrdfld, swdetach, swclose
  INTEGER   stdout_interval
  INTEGER   sw_id, sd_id
  REAL(4)   a1, a2, a3

  CHARACTER(LEN=max_len)  ::  var_name
  CHARACTER(LEN=max_len)  ::  QA_var_name

  CHARACTER(LEN=max_len)    ::  input_file
  CHARACTER(LEN=max_len)    ::  sub_name

  sub_name = "SUB sfc_albedo_extract_sds_MOD04"

  sd_id = swath_info%sd_id
  sw_id = swath_info%sw_id
  cHHMM = swath_info%cHHMM
  input_file = swath_info%ifile

! nx = across_swath (E-W), ny = along_swath (N-S)
  CALL dump_lonlat(sw_id, nx, ny,   &   ! IN
        L2_xlon, L2_ylat)   ! OUT

! +++ Aerosol Optical Depth (AOD; 2-byte integer; scale_factor = 0.001)
  edge(1) = nx
  edge(2) = ny

!!!  target_field_name = "Optical_Depth_Land_And_Ocean"
  target_field_name = "Surface_Reflectance_Land"
  statn = swrdfld(sw_id, TRIM(target_field_name), start, stride, edge, i2dummy)
  IF (statn /= 0) CALL swrdfld_err_msg(statn, TRIM(target_field_name) )

!!! Convert into float data for AOD (normal range = 0 - 3)
  CALL get_scale_offset(TRIM(input_file),  TRIM(target_field_name),    &    ! IN
      dscale_factor, doffset, dmissing)   ! OUT
  CALL convert_short2float(nx, ny, dscale_factor, doffset, i2dummy,    &    ! IN
      L2_albedo)   ! OUT

! +++ TEST output
  stdout_interval = 20
  DO iy = 1, ny, stdout_interval
    ix = 1 + nx / 2    ! nadir + 1 only
    a1 = L2_albedo(ix, iy)
    IF ( a1 > -900 ) THEN
      WRITE(*, '(a, a,2x, 2i4, a, 2f12.3, f12.3)') &
          TRIM(sub_name)//": HHMM, ix, iy = ", cHHMM, ix, iy,    &
          "; lon, lat, albedo = ", L2_xlon(ix, iy), L2_ylat(ix, iy),   &
          L2_albedo(ix, iy)
    END IF
  END DO    ! iy

END SUBROUTINE sfc_albedo_extract_sds_MOD04
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
SUBROUTINE close_sds(sw_id, sd_id)    ! IN
  INTEGER, INTENT(IN) :: sw_id, sd_id
  INTEGER   swdetach, swclose

! +++ Final clean-up
  statn = swdetach(sw_id)
  IF (statn /= 0)   &
    PRINT *, "Error detaching from input file. sw_id =  ", sw_id

  statn = swclose(sd_id)
  IF (statn /=  0)   &
    PRINT *, "Error closing input file. sd_id = ", sd_id

END SUBROUTINE close_sds
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE get_scale_offset(filename,  field_name,    &   ! IN
      dscale_factor, add_offset, Fill_Value)    ! OUT
  INCLUDE 'hdf.f90'
  INCLUDE 'mffunc.f90'
!!!!!! INTEGER(SP),  INTENT(IN)  :: sd_id    ! file ID
  CHARACTER(LEN=*), INTENT(IN)  ::  filename, field_name

! +++ Set to double precision
  REAL(KIND=8), INTENT(OUT)   :: dscale_factor
  REAL(KIND=8), INTENT(OUT)   :: add_offset
  REAL(KIND=8), INTENT(OUT)   :: Fill_Value

! +++ To handle attribute
  CHARACTER(LEN=32) :: attribute(3)
  INTEGER   attr_ix(3)

  INTEGER   stat
  INTEGER   file_id     ! File ID
  INTEGER   field_ix, field_id    ! Data Field number and its ID

! INTEGER   sfstart, sfendacc, sfend
! INTEGER   sfn2index, sfselect, sfrnatt

  CHARACTER(LEN=255) :: sub_name

  sub_name = "SUB get_scale_offset"

  attr_ix(:) = 0

#if defined DEBUG
  WRITE(*, *)
  WRITE(*, '(2x,a, 2i5)')    &
      "Length of filename = ", LEN(filename), LEN_TRIM(filename)
  WRITE(*, '(4x,a, a)')    &
      "Filename = ", filename
  WRITE(*, '(6x,a, a)')    &
      "Field name = ", field_name
#endif

! +++ attribute name setting (for CloudSat)
  attribute(1) = 'scale_factor'
  attribute(2) = 'add_offset'
  attribute(3) = '_FillValue'

! +++ open the hdf file
  file_id = sfstart(filename, DFACC_RDONLY)
  if (file_id == -1 ) THEN
    WRITE(*, *) TRIM(sub_name)//': CANNOT OPEN FILE', file_id 
    STOP    
  END IF

! +++ select the data field
  field_ix = sfn2index(file_id, field_name)
  if (field_ix == -1 ) THEN
    WRITE(*, *) TRIM(sub_name)//': CANNOT SELECT DATA FIELD', field_ix
    STOP    
  END IF
  field_id = sfselect(file_id, field_ix)

! +++ read the factor (or scale_factor) attribute
  attr_ix(1) = sffattr(field_id, attribute(1))
  IF (attr_ix(1) /= -1 ) THEN   ! found
    stat = sfrnatt(field_id, attr_ix(1), dscale_factor)
  ELSE                        ! not found, then defalut value was set.
    dscale_factor = 1.0
  END IF 
#ifdef DEBUG
  WRITE(*, '(4x,a, f12.4)') TRIM(attribute(1))//': ', dscale_factor
#endif

! +++ read the offset attribute
  attr_ix(2) = sffattr(field_id, attribute(2))
  IF (attr_ix(2) /= -1 ) THEN   ! found
    stat = sfrnatt(field_id, attr_ix(2), add_offset)
  ELSE                        ! not found, then defalut value was set.
    add_offset = 0.0
  END IF 
#ifdef DEBUG
  WRITE(*, '(4x,a, f12.4)') TRIM(attribute(2))//': ', add_offset
#endif

! +++ read the missing attribute
  attr_ix(3) = sffattr(field_id, attribute(3))
  stat = sfrnatt(field_id, attr_ix(3), Fill_Value)
#ifdef DEBUG
  WRITE(*, '(4x,a,a, f12.4)') TRIM(attribute(3)), ': ', Fill_Value
#endif
!!  WRITE(*, *) &
!!        TRIM(attribute(3)), ': ', Fill_Value

! +++ terminate access and close file
  stat = sfendacc(field_id)
  stat = sfend(file_id)

  END SUBROUTINE get_scale_offset
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE dump_lonlat(sw_id, nx, ny,   &   ! IN
        out_lon, out_lat)   ! OUT
  INTEGER(SP),  INTENT(IN)  :: sw_id
  INTEGER(SP),  INTENT(IN)  :: nx, ny        ! nx = across_swath (E-W), ny = along_swath (N-S)
  REAL(SP),     INTENT(OUT) :: out_lon(nx, ny), out_lat(nx, ny)

  !!! local variables
  REAL(SP), DIMENSION(nx, ny) :: d1, d2
  INTEGER swrdfld

! +++ Longitude
  edge(1) = nx  ! CELL_ACROSS_SWATH
  edge(2) = ny  ! CELL_ALONG_SWATH
  statn = swrdfld(sw_id, "Longitude", start, stride, edge, d1)
  IF (statn /= 0) PRINT *, "Error ", statn, " reading field ", "Longitude"
  out_lon(:, :) = d1(:, :)

! +++ Latitude
  edge(1) = nx  ! CELL_ACROSS_SWATH
  edge(2) = ny  ! CELL_ALONG_SWATH
  statn = swrdfld(sw_id, "Latitude", start, stride, edge, d2)
  IF (statn /= 0) PRINT *, "Error ", statn, " reading field ", "Latitude"
  out_lat(:, :) = d2(:, :)

  END SUBROUTINE dump_lonlat
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE convert_short2float(nx, ny, sf, offset, dinput_2D,     &   ! IN
        doutput_2D)   ! OUT
  INTEGER(SP),  INTENT(IN)  :: nx, ny        ! nx = across_swath (E-W), ny = along_swath (N-S)
  REAL(DP),     INTENT(IN)  :: sf            ! scale factor
  REAL(DP),     INTENT(IN)  :: offset        ! add offset
  INTEGER(2),   INTENT(IN)  :: dinput_2D(nx, ny)    ! "short" integer (2-byte integer)

  REAL(SP),     INTENT(OUT) :: doutput_2D(nx, ny)   ! output

  INTEGER, PARAMETER :: thresh_err = -9990
  REAL, PARAMETER :: real_error = -9999.0

  DO iy = 1, ny
    DO ix = 1, nx
      IF ( dinput_2D(ix, iy) < thresh_err ) THEN
        doutput_2D(ix, iy) = real_error    ! set in the "settings" module
      ELSE
        doutput_2D(ix, iy) = sf * ( dinput_2D(ix, iy) - offset )
      END IF
    END DO    ! ix
  END DO    ! iy

  END SUBROUTINE convert_short2float
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
  SUBROUTINE swrdfld_err_msg(statn, target_name)
  INTEGER, INTENT(IN) :: statn
  CHARACTER(LEN=*), INTENT(IN) :: target_name

  WRITE(*, '(2x,a, i3, a, a)') &
      "Error ", statn, " reading field ", target_name

  END SUBROUTINE swrdfld_err_msg
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
END MODULE mod_sample_decode_L2
